import com.urbancode.anthill3.*
import com.urbancode.anthill3.plugin.scm.*

final def workDir = new File('.').canonicalFile
final def out = System.out
final def env = System.getenv()

/*
final def DATE_READ_1 = new java.text.SimpleDateFormat("E MMM dd HH:mm:ss zzz yyyy") // java.util.Date.toString()
final def DATE_READ_2 = new java.text.SimpleDateFormat("yyyy-MM-dd HH:mm:ss z") // ISO-like format
final def SVN_DATE = new java.text.SimpleDateFormat("yyyy-MM-dd HH:mm:ss.S Z")
SVN_DATE.timeZone = TimeZone.getTimeZone("GMT")
*/

//------------------------------------------------------------------------------
// GET ALL INPUT PARAMETERS
//------------------------------------------------------------------------------

final def ahpTool = new AHPTool();
final stepProps = ahpTool.getStepProperties()

final def cleanupOps   = stepProps['operations'].split(",") as List<String>
final boolean svnCleanup   = Boolean.valueOf(stepProps['svnCleanup']);
final def svnExe       = stepProps['source/repo/commandPath']?: 'svn'
final def srcName      = stepProps['source'];
final def dir          = new File(workDir, stepProps['source/dirOffset'] ?: '.').canonicalFile // get checkout directory

final def cmdHelper = new CommandHelper(dir);

//------------------------------------------------------------------------------
// PREPARE WORKING DIRECTORY
//------------------------------------------------------------------------------

// create the working directory
dir.mkdirs()
if (!dir.exists()) {
    throw new Exception("Could not create working directory ${workDir}")
}
else if (dir.isFile()) {
    throw new Exception("Specified working directory is a file!! ${workDir}")
}

//------------------------------------------------------------------------------
// PREPARE COMMAND LINE
//------------------------------------------------------------------------------

def del = { File file ->
    boolean success = false;
    if (file.isFile()) {
        success = file.delete()
    }
    else if (file.isDirectory()){
        success = file.deleteDir()
    }
    if (!success) {
        System.err.println("Failed to remove path $file")
    }
}

// All Files
if (cleanupOps.contains("allFiles")) {\
    println "Cleanup all files is selected (other operations not applicable)."
    println "Deleting directory contents for $dir"
    dir.listFiles().each(del);
    println "Done deleting directory contents for $dir"
}
else {

    if (!new File(dir, '.svn').exists()) {
        println "Directory $dir.path is not an svn workspace skipping cleanup";
    }
    else {
        if (svnCleanup) {
            def stOut = null;
            def stCmd = [svnExe, 'cleanup', dir];
            cmdHelper.runCommand('Performing "svn cleanup" operation on $dir', stCmd)
        }
        // Remove unversioned files
        if (cleanupOps.contains("unversionedFiles")) {
            def stOut = null;
            def stCmd = [svnExe, 'status'];
            cmdHelper.runCommand("Removing unversioned files from $dir", stCmd) { proc ->

                proc.out.close() // close stdin
                def errThread = proc.consumeProcessErrorStream(System.err);
                try {
                    stOut = proc.text;

                    // trim trailing whitespace in output, otherwise we may get extra consecuative line endings
                    println stOut.replaceFirst(~/\s+$/, '');
                }
                finally {
                    if (errThread != null) {
                        errThread.join();
                    }
                }
            }

            // find lines with unversioned files
            stOut.eachLine{ line ->
                // line starts with ?
                if (line =~ /^\?/) {
                    String fileName = line.replaceFirst(~/^\?       /, "");
                    File file = new File(dir, fileName);
                    del(file);
                }
            }

            println "Done deleting unversioned files"
        }

        // Revert files
        if (cleanupOps.contains("revertFiles")) {
            def rvt = [svnExe, 'revert', '--recursive', dir.canonicalPath];
            cmdHelper.runCommand("Reverting all files in $dir", rvt);
            println "Done reverting files"
        }

    }
}
