import com.urbancode.anthill3.*
import com.urbancode.anthill3.plugin.scm.*

final def workDir = new File('.').canonicalFile
final def out = System.out
final def env = System.getenv()

/*
final def DATE_READ_1 = new java.text.SimpleDateFormat("E MMM dd HH:mm:ss zzz yyyy") // java.util.Date.toString()
final def DATE_READ_2 = new java.text.SimpleDateFormat("yyyy-MM-dd HH:mm:ss z") // ISO-like format
final def SVN_DATE = new java.text.SimpleDateFormat("yyyy-MM-dd HH:mm:ss.S Z")
SVN_DATE.timeZone = TimeZone.getTimeZone("GMT")
*/

//------------------------------------------------------------------------------
// GET ALL INPUT PARAMETERS
//------------------------------------------------------------------------------

final def ahpTool = new AHPTool();
final stepProps = ahpTool.getStepProperties()

final def svnExe       = stepProps['source/repo/commandPath']?: 'svn'
final def srcName      = stepProps['source'];
final def dir          = new File(workDir, stepProps['source/dirOffset'] ?: '.').canonicalFile // get checkout directory

final def cmdHelper = new CommandHelper(dir);

//------------------------------------------------------------------------------
// PREPARE WORKING DIRECTORY
//------------------------------------------------------------------------------

// create the working directory
dir.mkdirs()
if (!dir.exists()) {
    throw new Exception("Could not create working directory ${workDir}")
}
else if (dir.isFile()) {
    throw new Exception("Specified working directory is a file!! ${workDir}")
}

//------------------------------------------------------------------------------
// PREPARE COMMAND LINE
//------------------------------------------------------------------------------

def stOut = null;
def stCmd = [svnExe, 'cleanup', dir];
cmdHelper.runCommand("Recursively cleaning up working copy, removing locks, resuming unfinished operations, etc.", stCmd) { proc ->

    proc.out.close() // close stdin
    def errThread = proc.consumeProcessErrorStream(System.err);
    try {
        stOut = proc.text;

        // trim trailing whitespace in output, otherwise we may get extra consecuative line endings
        println stOut.replaceFirst(~/\s+$/, '');
    }
    finally {
        if (errThread != null) {
            errThread.join();
        }
    }
}
