import com.urbancode.anthill3.*

final def workDir = new File('.').canonicalFile
final def out = System.out
final def env = System.getenv()

//------------------------------------------------------------------------------
// GET ALL INPUT PARAMETERS AND VALIDATE
//------------------------------------------------------------------------------
final def getInput = { value -> !value || value.startsWith('${') ? null : value }

final def ahptool = new AHPTool()
def props = ahptool.getStepProperties()

final def svnExe       = props['source/repo/commandPath']?: 'svn'

def svnRoot            = props['source/repo/repositoryUrl']
final def svnUsr       = props['source/repo/username']
final def svnPwd       = props['source/repo/password']
final def svnBranches  = props['source/projectBranchesPath']
final def svnTags      = props['source/projectTagsPath']

final def createBranch = props['createBranch']
final def createTag    = props['createTag']
final def message      = props['commitMessage']
final def noCache      = props['cache']

if (!svnRoot) {
    throw new Exception("An svn Repository Url must be specified")
}
else if (!svnRoot.startsWith('http') && !svnRoot.startsWith('svn:')) {
    // should we prepend 'http://' or throw exception ????
    //if (!(svnRoot =~ /^(http|https|svn):[/][/]/)) svnRoot = 'http://'+svnRoot
}

if (!svnRoot.endsWith('/')) {
    svnRoot += '/'
}

if (createBranch && createTag) {
    throw new Exception('Both a branch and tag can not be specified at the same time')
}

if (!createBranch && !createTag) {
    throw new Exception('Must specify either a branch or a tag')
}

//------------------------------------------------------------------------------
// PREPARE COMMAND LINE
//------------------------------------------------------------------------------

    def command = null
    if (noCache) {
        command = [svnExe, '--non-interactive', '--no-auth-cache', 'copy']
    }
    else {
        command = [svnExe, '--non-interactive', 'copy']
    }

    if (svnUsr) {
        command.addAll(['--username', svnUsr])

        if (svnPwd) {
            command.addAll(['--password', svnPwd])
        }
    }

    command.addAll(['--message', message])
    command.add('.') // label from workDir

    if (createBranch) {
        command.add(svnRoot + svnBranches + '/' + createBranch)
    }
    else if (createTag) {
        command.add(svnRoot + svnTags + '/' + createTag)
    }
    else {
        throw new IllegalStateException() // shouldn't happen
    }


//------------------------------------------------------------------------------
// EXECUTE
//------------------------------------------------------------------------------

    // prepare process and set environment: MUST cast command to String[] !!!
    def pb = new ProcessBuilder((String[])command).directory(workDir)

    // print out command info
    println("command line: ${command.join(' ')}")
    println('-------------------------------')

    // run the process
    def proc = pb.start()
    proc.consumeProcessOutput(out, out)
    proc.waitFor()

    // print results
    println('--------------------------------')
    println("svn exit code:${proc.exitValue()}")

    if (proc.exitValue()) {
      System.exit(proc.exitValue()) // System.exit to preserve exit code?
    }

System.exit(0) // System.exit to preserve exit code